/*
 * ============================================================================
 * GNU Lesser General Public License
 * ============================================================================
 *
 * Square Brackets - Java Array Framework.
 * Copyright (C) 2013  Beatgrid Media B.V.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 *
 * Beatgrid Media B.V.
 * Sisalbaan 5-A
 * 2352 AZ Leiderdorp
 * The Netherlands
 *
 * info@squarebrackets.org
 * http://squarebrackets.org
 */
package org.squarebrackets;

import javax.annotation.CheckForNull;
import javax.annotation.Nullable;
import java.util.Comparator;

/**
 * Interface for getting and setting array characteristics.
 *
 * @author Leon van Zantvoort
 */
public interface ArrayContext<E> extends ArrayCharacteristics, MutableArrayCharacteristics {

    /**
     * Returns the backing array (optional operation). Modifications to the backing array are reflected by the mutable
     * array.
     *
     * <p>The resulting object must be cast to the primitive or object type of the implementing class.</p>
     *
     * @return backing array.
     * @throws UnsupportedOperationException if the array on which {@code doAction} is invoked does not expose backing
     * array.
     */
    Object array();

    /**
     * Gets the comparator that is used for sorting this array, {@code null} if array is primitive array, or if array
     * sorted according to natural ordering of elements, or {@code null} if array is not sorted.
     *
     * @return a {@code Comparator}, or {@code null} if the elements are sorted in the natural order, or {@code null} if
     * array is not sorted.
     */
    @CheckForNull Comparator<? super E> getComparator();

    /**
     * Sets the comparator that is used for sorting this array, {@code null} if array is primitive array, or if array
     * sorted according to natural ordering of elements, or {@code null} if array is not sorted.
     *
     * @param comparator the {@code Comparator} used to compare array elements. A {@code null} value indicates that the
     *                   elements' <i>Comparable natural ordering</i> is be used.
     * @throws UnsupportedOperationException if the array on which {@code doAction} is invoked is a primitive array.
     */
    void setComparator(@Nullable Comparator<? super E> comparator);
}
